--
-- AnimatedBelt
-- Specialization for animted belts
--
-- @author  Stefan Maurus	
-- @date  2016-01-14

-- Copyright © Stefan Maurus, www.stefanmaurus.de


AnimatedBelt = {};

function AnimatedBelt.prerequisitesPresent(specializations)
    return true;
end;

function AnimatedBelt:load(savegame)
	self.setBeltAnimationTime = SpecializationUtil.callSpecializationsFunction("setBeltAnimationTime"); 

	local i = 0;
	self.beltAnimations = {};
	while true do
        local key = string.format("vehicle.beltAnimations.animation(%d)", i);
		if not hasXMLProperty(self.xmlFile, key) then
			break;
		end;
		
		local name = getXMLString(self.xmlFile, key.."#name");
		
		local stepCurve = AnimCurve:new(linearInterpolatorTransRotScale);
		
		local maxTime = 0;
		
		local j = 0;
		while true do
			local key2 = string.format(key..".steps.step(%d)", j);
			if not hasXMLProperty(self.xmlFile, key2) then
				break;
			end;
			
			local tx, ty, tz = Utils.getVectorFromString(getXMLString(self.xmlFile, key2.."#position"));
			local rx, ry, rz = Utils.getVectorFromString(getXMLString(self.xmlFile, key2.."#rotation"));
			local t = getXMLFloat(self.xmlFile, key2.."#time");
			maxTime = math.max(maxTime, t);
			
			stepCurve:addKeyframe({x=tx, y=ty, z=tz, rx=math.rad(rx), ry=math.rad(ry), rz=math.rad(rz), sx=0,sy=0,sz=0, time = t});
			j = j + 1;
		end;

		j = 0;
		local parts = {};
		while true do
			local key2 = string.format(key..".parts.part(%d)", j);
			if not hasXMLProperty(self.xmlFile, key2) then
				break;
			end;
			
			local index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key2.."#index"));
			local startTime = getXMLFloat(self.xmlFile, key2.."#start");
			
			table.insert(parts, {index=index, startTime=startTime});
			j = j + 1;
		end;
		
		self.beltAnimations[name] = {stepCurve=stepCurve, parts=parts, maxTime=maxTime};
		self:setBeltAnimationTime(name, 0);
		i = i + 1;
	end;
	
	
end;

function AnimatedBelt:delete()
end;

function AnimatedBelt:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AnimatedBelt:keyEvent(unicode, sym, modifier, isDown)
end;

function AnimatedBelt:setBeltAnimationTime(name, t) 
	local animation = self.beltAnimations[name];
	for i, part in pairs(animation.parts) do
		local curTime = t+part.startTime;
		if curTime > animation.maxTime then
			curTime = curTime - animation.maxTime;
		end;
		
		local x,y,z, rx,ry,rz, _,_,_ = animation.stepCurve:get(curTime);
		setTranslation(part.index, x,y,z);
		setRotation(part.index, rx,ry,rz);
	end;
end;

function AnimatedBelt:update(dt)
end;

function AnimatedBelt:updateTick(dt)
end;

function AnimatedBelt:draw()
end;