--
-- Merger
-- Specialization for Mergers
--
-- @author  Stefan Maurus
-- @date  2016-01-14

-- Copyright © Stefan Maurus, www.stefanmaurus.de

Merger = {};

function Merger.prerequisitesPresent(specializations)
    return true;
end;

function Merger:load(savegame)
    local xmlFile = self.xmlFile;
	self.setControlAnimationIndex = SpecializationUtil.callSpecializationsFunction("setControlAnimationIndex");
	self.setAnimationState = SpecializationUtil.callSpecializationsFunction("setAnimationState");
	self.setControlBeltIndex = SpecializationUtil.callSpecializationsFunction("setControlBeltIndex");
	self.updateMergerBelt_Pickup = SpecializationUtil.callSpecializationsFunction("updateMergerBelt_Pickup");
	self.updateShaderCombination = SpecializationUtil.callSpecializationsFunction("updateShaderCombination");
	self.setDropAreas = SpecializationUtil.callSpecializationsFunction("setDropAreas");
	self.setWorkAreasActive = SpecializationUtil.callSpecializationsFunction("setWorkAreasActive");
	self.setShaderAnimationVisibility = SpecializationUtil.callSpecializationsFunction("setShaderAnimationVisibility");
	self.updateShaderAnimations = SpecializationUtil.callSpecializationsFunction("updateShaderAnimations");
	self.setShaderFruitType = SpecializationUtil.callSpecializationsFunction("setShaderFruitType");
	self.setAnimTime = SpecializationUtil.callSpecializationsFunction("setAnimTime");
    self.drawMergeModeCollectorHelp = SpecializationUtil.callSpecializationsFunction("drawMergeModeCollectorHelp");
    self.drawMergeModeBeltHelp = SpecializationUtil.callSpecializationsFunction("drawMergeModeBeltHelp");

	self.setUnloading = SpecializationUtil.callSpecializationsFunction("setUnloading");

	self.applyInitialAnimation = Utils.overwrittenFunction(self.applyInitialAnimation, Merger.applyInitialAnimation);

	self.controlAnimationIndex = 1;
	self.controlBeltIndex = 1;
	self.animationState = 1;

    self.tipToGroundLineOffset = 0;

	self.numberOfAnimations = getXMLInt(xmlFile, "vehicle.merger.liftAnimations#count");

	self.liftAnimation = {}
	self.liftAnimation.animation = {}
	for i = 1, self.numberOfAnimations do
		self.liftAnimation.animation[i] = {}
		self.liftAnimation.animation[i].name = getXMLString(xmlFile, string.format("vehicle.merger.liftAnimations.animation%d#name", i));
		self.liftAnimation.animation[i].speed = getXMLFloat(xmlFile, string.format("vehicle.merger.liftAnimations.animation%d#speed", i));
		self.liftAnimation.animation[i].middleTime = getXMLFloat(xmlFile, string.format("vehicle.merger.liftAnimations.animation%d#middleTime", i));
		self.liftAnimation.animation[i].startTime = 1;
	end;

	self.collector = {};
	for i = 1, self.numberOfAnimations do
		self.collector[i] = {}
		self.collector[i].pickupAnimation = getXMLString(xmlFile, string.format("vehicle.merger.collectors.collector%d#pickupAnimation", i));
		self.collector[i].pickupSpeed = getXMLFloat(xmlFile, string.format("vehicle.merger.collectors.collector%d#pickupSpeed", i));
		self.collector[i].maxPickupPosition = getXMLFloat(xmlFile, string.format("vehicle.merger.collectors.collector%d#maxPickupPosition", i));
		self.collector[i].currentPickupSpeed = 0;
		self.collector[i].currentPickupPosition = 0;

		self.collector[i].beltScroller = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.merger.collectors.collector%d#beltScroller", i)));
		self.collector[i].maxShaderPosition = getXMLFloat(xmlFile, string.format("vehicle.merger.collectors.collector%d#maxShaderPosition", i));
		self.collector[i].beltSpeed = getXMLFloat(xmlFile, string.format("vehicle.merger.collectors.collector%d#beltSpeed", i));

		self.collector[i].currentBeltPosition = 0;
		self.collector[i].beltAnimationName = getXMLString(xmlFile, string.format("vehicle.merger.collectors.collector%d#beltAnimationName", i));
		self.collector[i].maxBeltPosition = getXMLFloat(xmlFile, string.format("vehicle.merger.collectors.collector%d#maxBeltPosition", i));

		self.collector[i].currentDirection = 1;
	end;

	local i = 0;
	self.animationCombinations = {};
	while true do
        local key = string.format("vehicle.merger.animationCombinations.combination(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local combination = {};
		for j = 1, self.numberOfAnimations do
			table.insert(combination, getXMLFloat(xmlFile, string.format(key.."#animation%d", j)));
		end;

		local overlay = createImageOverlay(self.baseDirectory..getXMLString(xmlFile, key.."#overlay"));
		local fillVolumes = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#fillVolumes"));

		table.insert(self.animationCombinations, {combination=combination, overlay=overlay, fillVolumes=fillVolumes});
		i = i + 1;
	end;

	local i = 0;
	self.beltDirectionCombinations = {};
	while true do
        local key = string.format("vehicle.merger.beltDirectionCombinations.combination(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local combination = {};
		for j = 1, self.numberOfAnimations do
			table.insert(combination, getXMLFloat(xmlFile, string.format(key.."#animation%d", j)));
		end;

		local overlay = createImageOverlay(self.baseDirectory..getXMLString(xmlFile, key.."#overlay"));
		local allowAi = Utils.getNoNil(getXMLBool(xmlFile, key.."#allowAi"), true);

		table.insert(self.beltDirectionCombinations, {combination=combination, overlay=overlay, allowAi=allowAi});
		i = i + 1;
	end;

    local uiScale = g_gameSettings:getValue("uiScale");
	local _, height = getNormalizedScreenValues(0, 100*uiScale);
	local _, headlineOffset = getNormalizedScreenValues(0, 20*uiScale);
	local _, buttonOffset = getNormalizedScreenValues(0, 26*uiScale);
	local _, iconOffset = getNormalizedScreenValues(0, 85*uiScale);

	self.overlay = {};
	self.overlay.height = height;
	self.overlay.headlineOffset = headlineOffset;
	self.overlay.buttonOffset = buttonOffset;
	self.overlay.iconOffset = iconOffset;

	local i = 0;
	self.shaderCombinations = {};
	while true do
        local key = string.format("vehicle.merger.shaderAnimations.combination(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local lower = {};
		for j = 1, self.numberOfAnimations do
			table.insert(lower, getXMLInt(xmlFile, string.format(key.."#lower%d", j)));
		end;

		local direction = {};
		for j = 1, self.numberOfAnimations do
			table.insert(direction, getXMLInt(xmlFile, string.format(key.."#dir%d", j)));
		end;

		local drop = {};
		for j = 1, self.numberOfAnimations do
			table.insert(drop, getXMLInt(xmlFile, string.format(key.."#drop%d", j))+1);
		end;

		local indices = Utils.getVectorNFromString(getXMLString(xmlFile, key.. "#indices"));

		table.insert(self.shaderCombinations, {lower=lower, direction=direction, drop=drop, indices=indices});
		i = i + 1;
	end;

	self.activeShaderCombination = getXMLInt(xmlFile, "vehicle.merger.shaderAnimations#start");

	local i = 0;
	self.shaderAnimations = {};
	while true do
        local key = string.format("vehicle.merger.shaderAnimations.shaderAnimation(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		local area = getXMLInt(xmlFile, key.."#areaIndex");
		local length = getXMLInt(xmlFile, key.."#length");
		local usedToLoad = Utils.getNoNil(getXMLBool(xmlFile, key.."#usedToLoad"), false);
		local startFadeOut = Utils.getNoNil(getXMLFloat(xmlFile, key.."#startFadeOut"), 0.5);
		local fadeInWhileLoading = getXMLFloat(xmlFile, key.."#fadeInWhileLoading");

		table.insert(self.shaderAnimations, {index=index, area=area, length=length, usedToLoad=usedToLoad, fadeInWhileLoading=fadeInWhileLoading, lastLoad=0, fadeIn=-0.7, fadeOut=startFadeOut, startFadeOut=startFadeOut, chargeTime=0});
		i = i + 1;
	end;

	local i = 0;
	self.beltUnloadingCombinations = {};
	while true do
        local key = string.format("vehicle.merger.beltUnloadingCombinations.combination(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local currentAnimationCombination = getXMLInt(xmlFile, key.."#currentAnimationCombination");
		local direction = getXMLInt(xmlFile, key.."#direction");
		local shaderAnimation = getXMLInt(xmlFile, key.."#shaderAnimation")+1;
		local dropArea = getXMLInt(xmlFile, key.."#dropArea")+1;

		self.beltUnloadingCombinations[(currentAnimationCombination+1)*direction] = {shaderAnimation=shaderAnimation, dropArea=dropArea};
		i = i + 1;
	end;

	for i = 1, self.numberOfAnimations do
		self.groundReferenceNodes[i].threshold_bac = self.groundReferenceNodes[i].threshold;
		self.groundReferenceNodes[i].threshold = -2;
	end;

	self:setControlBeltIndex(1);
	self:setControlAnimationIndex(1, nil, true);

	self.overlayIsActive = false;

	self.unloading = false;
	self.unloadingDirection = 1;

	self.accumulatedFruitTypeBackup = FruitUtil.FRUITTYPE_UNKNOWN;
end;

function Merger:postLoad(savegame)
    if savegame ~= nil and not savegame.resetVehicles then
        self:setControlAnimationIndex(Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key.."#controlAnimationIndex"), self.controlAnimationIndex), nil, true);
        self:setControlBeltIndex(Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key.."#controlBeltIndex"), self.controlBeltIndex));

        for i=1, self.numberOfAnimations do
            local t = getXMLFloat(savegame.xmlFile, savegame.key.."#liftAnimationTime"..i);
            if t ~= nil and t ~= 1 and not resetVehicles then
                self.liftAnimation.animation[i].startTime = t;
            end;

            self:setAnimTime(i, self.liftAnimation.animation[i].startTime, true);
        end;
    end;
end

function Merger:delete()
end;

function Merger:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Merger:keyEvent(unicode, sym, modifier, isDown)
end;

function Merger:setAnimTime(animation, t, noEventSend)
	local anim = self.liftAnimation.animation[animation].name;
	local cur = self:getAnimationTime(anim);

	if cur ~= t then
		local dir = 1;
		if cur > t then
			dir = -1;
		end;

		self:playAnimation(anim, dir, cur, noEventSend);
		self:setAnimationStopTime(anim, t);
		AnimatedVehicle.updateAnimationByName(self, anim, 99999999);
	end;

	--if self.isServer then
		for _, tool in pairs(self.movingTools) do
			Cylindered.setDirty(self, tool);
		end
	--end;

	for _, part in pairs(self.movingParts) do
		Cylindered.setDirty(self, part);
	end

	self:updateCylinderedInitial(true);
end;

function Merger:setRelativePosition(positionX, offsetY, positionZ, yRot)
	for i = 1, self.numberOfAnimations do
		self:setAnimTime(i, self.liftAnimation.animation[i].startTime, true);
	end;
end

function Merger:applyInitialAnimation(superFunc)
	for i = 1, self.numberOfAnimations do
		self:setAnimTime(i, self.liftAnimation.animation[i].startTime, true);
	end;
    if superFunc ~= nil then
        superFunc(self);
    end
end

function Merger:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'controlAnimationIndex="'..self.controlAnimationIndex..'" controlBeltIndex="'..self.controlBeltIndex..'"';

	for i=1, self.numberOfAnimations do
		local t = self:getAnimationTime(self.liftAnimation.animation[i].name);
		if t ~= 0 and t~= 1 then
			t = (math.ceil(t*1000)+1)/1000;
		end;
		attributes = attributes..' liftAnimationTime'..i..'="'..t..'"'
	end;

	return attributes, nil;
end;

function Merger:readStream(streamId, connection)
	self:setControlAnimationIndex(streamReadInt8(streamId), true, true);
	self:setControlBeltIndex(streamReadInt8(streamId), true);
	self:setUnloading(streamReadBool(streamId), streamReadInt8(streamId), true);

	for i=1, self.numberOfAnimations do
		local t = streamReadFloat32(streamId);

		if t ~= nil then
			self:setAnimTime(i, t, true);
		end;
	end;
end;

function Merger:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.controlAnimationIndex);
	streamWriteInt8(streamId, self.controlBeltIndex);
	streamWriteBool(streamId, self.unloading);
	streamWriteInt8(streamId, self.unloadingDirection);

	for i=1, self.numberOfAnimations do
		local t = self:getAnimationTime(self.liftAnimation.animation[i].name);
		if t ~= 0 and t~= 1 then
			t = (math.ceil(t*1000)+1)/1000;
		end;
		streamWriteFloat32(streamId, t);
	end;
end;

function Merger:setControlAnimationIndex(index, noEventSend, noUpdate)
	if index > table.getn(self.animationCombinations) then
		index = 1;
	elseif index == 0 then
        index = table.getn(self.animationCombinations);
    end;

	self.controlAnimationIndex = index;

	if noUpdate == nil then
		for i, v in ipairs(self.animationCombinations[self.controlAnimationIndex].combination) do
			local anim = self.liftAnimation.animation[i];
			local time = self:getAnimationTime(anim.name);

			if v == 0 then
				self:playAnimation(anim.name, anim.speed, time);
				self:setAnimationStopTime(anim.name, 1);
			else
				if time < anim.middleTime then
					self:playAnimation(anim.name, anim.speed, time);
					self:setAnimationStopTime(anim.name, anim.middleTime);
				elseif time > anim.middleTime then
					self:playAnimation(anim.name, -anim.speed, time);
					self:setAnimationStopTime(anim.name, anim.middleTime);
				end;
			end;
		end;
	end;

	self:updateShaderCombination(true);

	for i, v in ipairs(self.fillVolumes) do
		local is = false;

		for j, b in ipairs(self.animationCombinations[self.controlAnimationIndex].fillVolumes) do
			if i == b then
				is = true;
			end;
		end;

		if is then
			setVisibility(v.baseNode, true);
		else
			setVisibility(v.baseNode, false);
		end;
	end;

	SetControlAnimationIndexEvent.sendEvent(self, index, noEventSend);
end;

function Merger:setControlBeltIndex(index, noEventSend)
	if index > table.getn(self.beltDirectionCombinations) then
		index = 1;
	elseif index == 0 then
        index = table.getn(self.beltDirectionCombinations);
    end;


	local combo = self.beltDirectionCombinations[index].combination;
	for i = 1, self.numberOfAnimations do
		self.collector[i].currentDirection = combo[i];
	end;

	self.controlBeltIndex = index;

	self:updateShaderCombination(true);

	SetControlBeltIndexEvent.sendEvent(self, index, noEventSend);
end;

function Merger:updateShaderCombination(force)
	force = Utils.getNoNil(force, false);
	for i, v in ipairs(self.shaderCombinations) do
		local is = true;
		for i = 1, self.numberOfAnimations do
			if v.lower[i] == 1 then
				if self:getAnimationTime(self.liftAnimation.animation[i].name) > self.liftAnimation.animation[i].middleTime then
					is = false;
				end;
			else
				if self:getAnimationTime(self.liftAnimation.animation[i].name) <= self.liftAnimation.animation[i].middleTime then
					is = false;
				end;
			end;

			if self.collector[i].currentDirection ~= v.direction[i] then
				is = false;
			end;
		end;

		if is then
			if i ~= self.activeShaderCombination then
				self.activeShaderCombination = i;
				self:setDropAreas();
				self:setShaderAnimationVisibility(self.activeShaderCombination, force);
			end;
			return;
		end;
	end;

	self.activeShaderCombination = table.getn(self.shaderAnimations);
	self:setShaderAnimationVisibility(self.activeShaderCombination, force);
	self:setDropAreas();
end;

function Merger:updateShaderAnimations(dt, loading)
	if loading == nil or not loading then
		self:setShaderAnimationVisibility(self.activeShaderCombination);

		for i, v in ipairs (self.shaderAnimations) do
			if getVisibility(v.index) then
				local typedWorkAreas = self:getTypedWorkAreas(WorkArea.areaTypeNameToInt["windrower"]);
				area = typedWorkAreas[v.area];

				local ret = 0;
				if self:getIsWorkAreaActive(area) then
                    local offset = 1;
					local xs, _, zs = localToWorld(area.start, 0, 0, offset);
					local xw, _, zw = localToWorld(area.width, 0, 0, offset);
					local xh, _, zh = localToWorld(area.height, 0, 0, 0);

                    ret = TipUtil.getFillLevelAtArea(FruitUtil.fruitTypeToWindrowFillType[self.accumulatedFruitType], xs,zs, xw,zw, xh,zh);
				end;

				if ret > 0 and self.movingDirection > 0 and self.lastSpeed*3600 > 0.5 then
					v.chargeTime = 500;
				end;
				v.chargeTime = math.max(v.chargeTime - dt, 0);

				if v.chargeTime > 0 then
					v.fadeIn = math.min(v.fadeIn + (0.0005*dt*v.length), 1);
					v.fadeOut = v.startFadeOut;
				else
					if v.fadeIn > -0.7 then
						v.fadeOut = math.max(v.fadeOut - (0.0005*dt*v.length), -1);
					end;
					if v.fadeOut == -1 then
						v.fadeIn = -0.7;
						v.fadeOut = v.startFadeOut;
					end;
				end;

				setShaderParameter(v.index, "fadeProgress", v.fadeIn, v.fadeOut, 0, 0, false);
				setShaderParameter(v.index, "UVScaleSpeed", 2*v.length, -0.7/v.length, 0, 0, false); --/v.length

				self:setShaderFruitType();
			end;
		end;
	else
		for i = 1, self.numberOfAnimations do
			if self.groundReferenceNodes[i].threshold == -3 then
				self.groundReferenceNodes[i].threshold = self.groundReferenceNodes[i].threshold_bac;

				local x,y,z = getWorldTranslation(self.groundReferenceNodes[i].node);
				local densityHeight, deltaHeight = TipUtil.getHeightAtWorldPos(x,y,z);
				WorkArea.updateGroundReferenceNode(self, self.groundReferenceNodes[i], x, y, z, densityHeight-deltaHeight, densityHeight);
			end;
		end;

		for i, v in ipairs (self.shaderAnimations) do
			if v.usedToLoad then
				local typedWorkAreas = self:getTypedWorkAreas(WorkArea.areaTypeNameToInt["windrower"]);
				area = typedWorkAreas[v.area];
				self.accumulatedFruitType = FruitUtil.fillTypeToFruitType[math.max(self.fillUnits[1].currentFillType, 1)];

				local ret = 0;
				if self:getIsWorkAreaActive(area) then
					local x, _, z = getWorldTranslation(area.start);
					local x1, _, z1 = getWorldTranslation(area.width);
					local x2, _, z2 = getWorldTranslation(area.height);

                    ret = TipUtil.getFillLevelAtArea(FruitUtil.fruitTypeToWindrowFillType[self.accumulatedFruitType], x,z, x1,z1, x2,z2);
				end;

				if ret > 0 and self.lastSpeed * 3600 > 1 and self.isTurnedOn then
					v.lastLoad = 400;
				else
					v.lastLoad = math.max(v.lastLoad - dt, 0);
				end;

				if v.lastLoad > 0 then
					setVisibility(v.index, true);

					setShaderParameter(v.index, "fadeProgress", v.fadeInWhileLoading, 1, 0, 0, false);
					setShaderParameter(v.index, "UVScaleSpeed", 2*v.length, -0.7/v.length, 0, 0, false);

					self:setShaderFruitType();
				else
					setVisibility(v.index, false);
				end;
			end;
		end;

		for i = 1, self.numberOfAnimations do
			self.groundReferenceNodes[i].threshold = -3;
		end;
	end;
end;

function Merger:setShaderFruitType()
	if self.accumulatedFruitType == FruitUtil.fillTypeToFruitType[FillUtil.fillTypeNameToInt["grass_windrow"]] or self.accumulatedFruitType == FruitUtil.fillTypeToFruitType[FillUtil.fillTypeNameToInt["dryGrass_windrow"]] then
		for i, v in ipairs (self.shaderAnimations) do
			setShaderParameter(v.index, "fruitType", 1, 0, 0, 0, false);
		end;
	else
		for i, v in ipairs (self.shaderAnimations) do
			setShaderParameter(v.index, "fruitType", 0, 0, 0, 0, false);
		end;
	end;
end;

function Merger:setShaderAnimationVisibility(m, force)
	for i, v in ipairs (self.shaderAnimations) do
		if (v.fadeIn == -0.7 and v.fadeOut == v.startFadeOut) or force then
			setVisibility(v.index, false);
			v.fadeIn = -0.7;
			v.fadeOut = v.startFadeOut;
			v.chargeTime = 0;

			setShaderParameter(v.index, "fadeProgress", v.fadeIn, v.fadeOut, 0, 0, false);
		end;
	end;

	for j, n in ipairs (self.shaderCombinations[m].indices) do
		setVisibility(self.shaderAnimations[n+1].index, true);
	end;
end;

function Merger:setDropAreas()
	local combination = self.shaderCombinations[self.activeShaderCombination];
	local typedWorkAreas = self:getTypedWorkAreas(WorkArea.areaTypeNameToInt["windrower"]);
	for i = 1, self.numberOfAnimations do
		typedWorkAreas[i].dropAreaIndex = combination.drop[i];
	end;
end;

function Merger:setWorkAreasActive()
	for i = 1, self.numberOfAnimations do
		if self:getAnimationTime(self.liftAnimation.animation[i].name) == 0 then
			self.groundReferenceNodes[i].threshold = self.groundReferenceNodes[i].threshold_bac;
		else
			self.groundReferenceNodes[i].threshold = -2;
		end;
		if self.controlBeltIndex == 5 then
			self.groundReferenceNodes[i].threshold = -3;
		end;
	end;
end;

function Merger:setAnimationState(input, noEventSend)
	self.animationState = input;

	local combo = self.animationCombinations[self.controlAnimationIndex].combination;
	for i, v in ipairs(combo) do
		if v == 1 then
			local anim = self.liftAnimation.animation[i];
			local time = self:getAnimationTime(anim.name);
			if input == 0 then
				if time < anim.middleTime then
					self:playAnimation(anim.name, anim.speed, time);
					self:setAnimationStopTime(anim.name, anim.middleTime);

					if self.attacherVehicle ~= nil then
						self.attacherVehicle:setJointMoveDown(self.attacherVehicle.selectedImplement.jointDescIndex, false, false);
					end;
				else
					self:playAnimation(anim.name, -anim.speed, time);
					self:setAnimationStopTime(anim.name, 0);
					if self.attacherVehicle ~= nil then
						self.attacherVehicle:setJointMoveDown(self.attacherVehicle.selectedImplement.jointDescIndex, true, false);
					end;
				end;
			elseif input == 1 then
				if time > anim.middleTime then
					self:playAnimation(anim.name, -anim.speed, time);
					self:setAnimationStopTime(anim.name, anim.middleTime);
				else
					self:playAnimation(anim.name, anim.speed, time);
					self:setAnimationStopTime(anim.name, 1);
				end;
			end;
		end;
	end;

	SetAnimationStateEvent.sendEvent(self, self.animationState, noEventSend);
end;

function Merger:setUnloading(state, direction, noEventSend)
	if not state then
		self.activeShaderCombination = table.getn(self.shaderAnimations);
		self:setShaderAnimationVisibility(self.activeShaderCombination, true);
		self:setDropAreas();
	end;

	if not self.unloading then
		self.unloadingDirection = Utils.getNoNil(direction, self.unloadingDirection);

		for i=1, 3 do
			self:updateMergerBelt_Pickup(i, 0, 0);
		end;
	end;

	self.unloading = Utils.getNoNil(state, self.unloading);

	SetUnloadingEvent.sendEvent(self, self.unloading, self.unloadingDirection, noEventSend);
end;

function Merger:update(dt)
	if self.controlBeltIndex ~= 5 and self.isClient then
		self:updateShaderAnimations(dt);
	end;

	if self:getIsActiveForInput() then
		local isPlaying = false;

		for i = 1, self.numberOfAnimations do
			if self:getIsAnimationPlaying(self.liftAnimation.animation[i].name) then
				isPlaying = true;
				break;
			end;
		end;

		if self:getFillLevel() == 0 then
			if InputBinding.hasEvent(InputBinding.MERGER_SWITCHANIMATION_BACK) then
				self:setControlAnimationIndex(self.controlAnimationIndex-1);
            else
                if InputBinding.hasEvent(InputBinding.MERGER_SWITCHANIMATION) then
                    self:setControlAnimationIndex(self.controlAnimationIndex+1);
                end;
			end;

			if InputBinding.hasEvent(InputBinding.MERGER_SWITCHBELT_BACK) then
				self:setControlBeltIndex(self.controlBeltIndex-1);
            else
                if InputBinding.hasEvent(InputBinding.MERGER_SWITCHBELT) then
                    self:setControlBeltIndex(self.controlBeltIndex+1);
                end;
			end;
		end;

		if not isPlaying then
			if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
				self:setAnimationState(0);
			end;

			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) and self:getFillLevel() == 0 then
				self:setAnimationState(1);
			end;
		end;

		if InputBinding.hasEvent(InputBinding.DISPLAY_OVERLAY) then
			self.overlayIsActive = not self.overlayIsActive;
		end;

		if self:getFillLevel() > 0 then
			if InputBinding.hasEvent(InputBinding.MERGER_UNLOAD_LEFT) then
				self:setUnloading(not self.unloading, 1);
			end;
			if InputBinding.hasEvent(InputBinding.MERGER_UNLOAD_RIGHT) then
				self:setUnloading(not self.unloading, -1);
			end;
		end;
	end;
end;

function Merger:updateTick(dt)
	if self.isServer then
		if self.accumulatedFruitType ~= self.accumulatedFruitTypeBackup then
			self.accumulatedFruitTypeBackup = self.accumulatedFruitType;
			SetAccumulatedFruitTypeEvent.sendEvent(self, self.accumulatedFruitType, noEventSend);
		end;

		local rootAttacherVehicle = self:getRootAttacherVehicle();

		if rootAttacherVehicle ~= nil then
			if rootAttacherVehicle.aiIsStarted then
				if not self.beltDirectionCombinations[self.controlBeltIndex].allowAi then
					self:setControlBeltIndex(self.controlBeltIndex+1);
				end;
			end;
		end;
	end;

	if self.isClient then
		self:updateShaderCombination();
	end;

	self:setWorkAreasActive();

	if self.unloading then
		if self:getIsTurnedOn() then
			self:setIsTurnedOn(false);
		end;

		local factor = math.min(self.lastSpeed * 3600 / 15, 1);
		local delta = dt * factor / 3;
        for i=1, 3 do
            self:setUnitFillLevel(i, math.max(self:getUnitFillLevel(i) - delta, 0), self.fillUnits[i].currentFillType);
        end;

		local unloadingCombo = self.beltUnloadingCombinations[self.controlAnimationIndex*self.unloadingDirection];

		local shaderAnimation = self.shaderAnimations[unloadingCombo.shaderAnimation];
		if factor > 0.05 then
			setVisibility(shaderAnimation.index, true);
			setShaderParameter(shaderAnimation.index, "UVScaleSpeed", 2*shaderAnimation.length, -0.9, 1, 0, false);
			setShaderParameter(shaderAnimation.index, "fadeProgress", 1, 0.1, 0, 0, false);

			self:setShaderFruitType();
		else
			setVisibility(shaderAnimation.index, false);
		end;

		for i = 1, self.numberOfAnimations do
			if self:getAnimationTime(self.liftAnimation.animation[i].name) <= self.liftAnimation.animation[i].middleTime then
				self:updateMergerBelt_Pickup(i, dt*self.unloadingDirection, dt*factor*self.unloadingDirection);
			end;
		end;

		local dropAreas = self:getTypedWorkAreas(WorkArea.AREATYPE_WINDROWERDROP);
		local dropArea = dropAreas[unloadingCombo.dropArea];

		local dx,dy,dz = getWorldTranslation(dropArea.start);
		local dx1,dy1,dz1 = getWorldTranslation(dropArea.width);
		local dx2,dy2,dz2 = getWorldTranslation(dropArea.height);

        local hx_2 = 0.5*(dx2 - dx);
        local hz_2 = 0.5*(dz2 - dz);
        local sx = dx + hx_2;
        local sz = dz + hz_2;
        local ex = dx1 + hx_2;
        local ez = dz1 + hz_2;
        local sy = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, sx,0,sz) + 1;
        local ey = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, ex,0,ez) + 1;

        local dropped, lineOffset = TipUtil.tipToGroundAroundLine(self, delta, self.fillUnits[1].currentFillType, sx,sy,sz, ex,ey,ez, 0, nil, self.mergerLineOffset, false, nil);
        self.mergerLineOffset = lineOffset;

		if self:getFillLevel() == 0 then
			self:setUnloading(false);
			setVisibility(shaderAnimation.index, false);
		end;
	end;

	if self.controlBeltIndex == 5 then
		self:updateShaderAnimations(dt, true);
	end;

	if self:getIsTurnedOn() then
		local allFolded = true;
		for i = 1, self.numberOfAnimations do
			if self:getAnimationTime(self.liftAnimation.animation[i].name) <= self.liftAnimation.animation[i].middleTime then
				self:updateMergerBelt_Pickup(i, dt);

				allFolded = false;
			else
				if self.collector[i].currentPickupSpeed > 0 then
					self:updateMergerBelt_Pickup(i, -dt);
				end;
			end;
		end;

		for i = 1, self.numberOfAnimations do
			if self.groundReferenceNodes[i].threshold == -3 then
				self.groundReferenceNodes[i].threshold = self.groundReferenceNodes[i].threshold_bac;

				local x,y,z = getWorldTranslation(self.groundReferenceNodes[i].node);
				local densityHeight, deltaHeight = TipUtil.getHeightAtWorldPos(x,y,z);
				WorkArea.updateGroundReferenceNode(self, self.groundReferenceNodes[i], x, y, z, densityHeight-deltaHeight, densityHeight);
			end;
		end;

		if self.controlBeltIndex == 5 then
            local hasGroundContact, typedWorkAreas = self:getIsTypedWorkAreaActive(WorkArea.AREATYPE_WINDROWER);
			self.hasGroundContact = hasGroundContact;

			if hasGroundContact and self.lastSpeed * 3600 > 1 then
				self.isWindrowerSpeedLimitActive = true;

				for k, workArea in pairs(typedWorkAreas) do
					if self:getIsWorkAreaActive(workArea) then
						local x,_,z = getWorldTranslation(workArea.start);
						local x1,_,z1 = getWorldTranslation(workArea.width);
						local x2,_,z2 = getWorldTranslation(workArea.height);

						for i, v in pairs(self.fillUnits[1].fillTypes) do
							local fruitType = FruitUtil.fillTypeToFruitType[i];
                            if fruitType ~= nil then
                                local amount = TipUtil.removeFromGroundByArea(x, z, x1, z1, x2, z2, FruitUtil.fruitTypeToWindrowFillType[fruitType]); --Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);

                                if self.isServer then
                                    if amount > 0 then
                                        self:setUnitFillLevel(k, self:getUnitFillLevel(k) + amount, i, true, self.fillVolumeLoadInfos[k]);
                                    end;
                                end;
                            end;
						end;
					end;
				end;
			end;

			for i = 1, self.numberOfAnimations do
				self.groundReferenceNodes[i].threshold = -3;
			end;
		end;

		if allFolded then
			self:setIsTurnedOn(false);
		end;
	elseif not self.unloading and not self:getIsTurnedOn() then
		for i = 1, self.numberOfAnimations do
			if self.collector[i].currentPickupSpeed > 0 then
				self:updateMergerBelt_Pickup(i, -dt);
			end;
		end;
	end;
end;

function Merger:updateMergerBelt_Pickup(i, dt, fixedSpeed)
	if fixedSpeed ~= nil then
		self.collector[i].currentPickupSpeed = fixedSpeed * 0.03;

		self.collector[i].currentBeltPosition = self.collector[i].currentBeltPosition + (math.abs(dt) * 0.001 * self.collector[i].currentPickupSpeed * self.collector[i].beltSpeed);
	else
		self.collector[i].currentPickupSpeed = math.max(math.min(self.collector[i].currentPickupSpeed + (dt * 0.001), 1), 0);

		self.collector[i].currentPickupPosition = self.collector[i].currentPickupPosition + self.collector[i].currentPickupSpeed*0.25;

		if self.collector[i].currentPickupPosition > self.collector[i].maxPickupPosition then
			self.collector[i].currentPickupPosition = self.collector[i].currentPickupPosition - self.collector[i].maxPickupPosition;
		end;

		self:setBeltAnimationTime(self.collector[i].pickupAnimation, self.collector[i].currentPickupPosition);

		self.collector[i].currentBeltPosition = self.collector[i].currentBeltPosition + (math.abs(dt) * 0.001 * self.collector[i].currentPickupSpeed * self.collector[i].beltSpeed * self.collector[i].currentDirection);
	end;

	if self.collector[i].currentBeltPosition > self.collector[i].maxBeltPosition then
		self.collector[i].currentBeltPosition = 0;
	end;

	if self.collector[i].currentBeltPosition < 0 then
		self.collector[i].currentBeltPosition = self.collector[i].maxBeltPosition;
	end;

	local shaderPosition = (self.collector[i].currentBeltPosition%0.25)/2.5;
	setShaderParameter(self.collector[i].beltScroller, "offsetUV", 0, shaderPosition, 0, 0, false);

	self:setBeltAnimationTime(self.collector[i].beltAnimationName, self.collector[i].currentBeltPosition);
end;

function Merger:drawMergeModeCollectorHelp(posY, param)
    Merger.drawMergeModeHelp(self, "ANIMATION", posY, param)
end

function Merger:drawMergeModeBeltHelp(posY, param)
    Merger.drawMergeModeHelp(self, "BELT", posY, param)
end

function Merger:drawMergeModeHelp(type, posY, param)
    if g_currentMission.inGameMessage:getIsVisible() then return; end

    local posX = g_currentMission.helpBoxTextPos1X;
    local posX2 = g_currentMission.helpBoxTextPos2X;

	-- render headline
    setTextColor(g_currentMission.helpBoxTextColor[1], g_currentMission.helpBoxTextColor[2], g_currentMission.helpBoxTextColor[3], g_currentMission.helpBoxTextColor[4]);

    setTextBold(true);
	setTextAlignment(RenderText.ALIGN_CENTER);
    renderText(g_safeFrameOffsetX + g_currentMission.helpBoxWidth * 0.5, posY - self.overlay.headlineOffset, g_currentMission.helpBoxTextSize, g_i18n:getText("MERGER_"..type.."COMBINATION"));
    setTextBold(false);

	-- render bottons
	local buttonPosY = g_currentMission.helpBoxContentOverlay.y + g_currentMission.helpBoxContentOverlay.height - g_currentMission.helpBoxContentHeight*0.5

    local control1 = InputBinding.getControllerSymbolOverlays(InputBinding["MERGER_SWITCH"..type.."_BACK"], nil, nil, true)
    local control2 = InputBinding.getControllerSymbolOverlays(InputBinding["MERGER_SWITCH"..type], nil, nil, true)

	for _, key in pairs(control1.keys) do
		posX = posX + g_currentMission.buttonOverlay:renderButton(key, posX, posY - self.overlay.buttonOffset, g_currentMission.buttonSizeY);
	end

	local width = 0;
	for _, key in pairs(control2.keys) do
		local buttonWidth = g_currentMission.buttonOverlay:getButtonWidth(key, g_currentMission.buttonSizeY);
		width = width + buttonWidth;
	end

	local offset = g_currentMission.helpBoxTextPos1X-g_safeFrameOffsetX;
	posX = g_safeFrameOffsetX+g_currentMission.helpBoxWidth-offset-width;
	for _, key in pairs(control2.keys) do
		posX = posX + g_currentMission.buttonOverlay:renderButton(key, posX, posY - self.overlay.buttonOffset, g_currentMission.buttonSizeY);
	end

	-- render icons
	local combination = type == "ANIMATION" and {self.animationCombinations, self.controlAnimationIndex} or {self.beltDirectionCombinations, self.controlBeltIndex};

	local index1 = combination[2]-1;
	if index1 < 1 then index1 = #combination[1] end;
	local index2 = combination[2];
	local index3 = combination[2]+1;
	if index3 > #combination[1] then index3 = 1 end;

	local renderModeOverlay = function(combination, index, posY, selected)
		local ratioX, ratioY = getNormalizedScreenValues(2, 1);
		local ratio = ratioY/ratioX;

		local section = g_currentMission.helpBoxWidth / 3;
		local iconWidth = 0.625;

		local targetWidth = section - g_currentMission.helpBoxTextSize;
		local targetHeight = targetWidth * ratio;

		local posX = section * index - section * 0.5;
		posX = posX - targetWidth / 2;

		setOverlayColor(combination.overlay, 1, 1, 1, 1);

		if not selected then
			posX = posX + targetWidth * 0.1;
			targetWidth, targetHeight = targetWidth * 0.8, targetHeight * 0.8;

			setOverlayColor(combination.overlay, 1, 1, 1, 0.6);
		end;

		renderOverlay(combination.overlay, g_safeFrameOffsetX + posX, posY, targetWidth/iconWidth, targetHeight/iconWidth);
	end;

	renderModeOverlay(combination[1][index1], 1, posY - self.overlay.iconOffset, false)
	renderModeOverlay(combination[1][index2], 2, posY - self.overlay.iconOffset, true)
	renderModeOverlay(combination[1][index3], 3, posY - self.overlay.iconOffset, false)
end

function Merger:draw()
	if self.overlayIsActive then
		if self:getIsActiveForInput(true) then
			g_currentMission:addHelpTextFunction(self.drawMergeModeCollectorHelp, self, self.overlay.height, nil);
			g_currentMission:addHelpTextFunction(self.drawMergeModeBeltHelp, self, self.overlay.height, nil);
		end
	end;

	if self:getFillLevel() > 0 then
		if not self.unloading then
			g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_UNLOAD_LEFT"), InputBinding.MERGER_UNLOAD_LEFT, nil, GS_PRIO_HIGH);
			g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_UNLOAD_RIGHT"), InputBinding.MERGER_UNLOAD_RIGHT, nil, GS_PRIO_HIGH);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_UNLOAD_STOP"), InputBinding.MERGER_UNLOAD_RIGHT, nil, GS_PRIO_HIGH);
		end;
	end;

	g_currentMission:addHelpButtonText(g_i18n:getText("DISPLAY_OVERLAY"), InputBinding.DISPLAY_OVERLAY, nil, GS_PRIO_HIGH);

	local currentState = 1;
	local combo = self.animationCombinations[self.controlAnimationIndex].combination;
	for i, v in ipairs(combo) do
		if v == 1 then
			local anim = self.liftAnimation.animation[i];
			if self:getAnimationTime(anim.name) == 1 then
				currentState = 2;
			elseif self:getAnimationTime(anim.name) == 0 then
				currentState = 0;
			end;
			break;
		end;
	end;

	if currentState == 0 then
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_LIFT"), InputBinding.LOWER_IMPLEMENT, nil, GS_PRIO_HIGH);
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_FOLD"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
	elseif currentState == 1 then
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_LOWER"), InputBinding.LOWER_IMPLEMENT, nil, GS_PRIO_HIGH);
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_FOLD"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
	elseif currentState == 2 then
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_LOWER"), InputBinding.LOWER_IMPLEMENT, nil, GS_PRIO_HIGH);
		g_currentMission:addHelpButtonText(g_i18n:getText("MERGER_UNFOLD"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
	end;
end;

function Merger:onAiRaise()
	local isLowered = true;

	local combo = self.animationCombinations[self.controlAnimationIndex].combination;
	for i, v in ipairs(combo) do
		if v == 1 then
			local anim = self.liftAnimation.animation[i];
			local time = self:getAnimationTime(anim.name);
			if time >= anim.middleTime then
				isLowered = false;
			end;
		end;
	end;

	if isLowered then
		self:setAnimationState(0);
	end;
end;

function Merger:onAiTurnOn()
	self:onAiLower();
end;

function Merger:onAiLower()
	local isLowered = true;

	local combo = self.animationCombinations[self.controlAnimationIndex].combination;
	for i, v in ipairs(combo) do
		if v == 1 then
			local anim = self.liftAnimation.animation[i];
			local time = self:getAnimationTime(anim.name);
			if time >= anim.middleTime then
				isLowered = false;
			end;
		end;
	end;

	if not isLowered then
		self:setAnimationState(0);
	end;
end;

SetControlAnimationIndexEvent = {};
SetControlAnimationIndexEvent_mt = Class(SetControlAnimationIndexEvent, Event);

InitEventClass(SetControlAnimationIndexEvent, "SetControlAnimationIndexEvent");

function SetControlAnimationIndexEvent:emptyNew()
    local self = Event:new(SetControlAnimationIndexEvent_mt);
    self.className="SetControlAnimationIndexEvent";
    return self;
end;

function SetControlAnimationIndexEvent:new(vehicle, index)
    local self = SetControlAnimationIndexEvent:emptyNew()
    self.vehicle = vehicle;
	self.index = index;
    return self;
end;

function SetControlAnimationIndexEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);

	self.index = streamReadInt8(streamId);
	if self.vehicle ~= nil then
		self.vehicle:setControlAnimationIndex(self.index, true);
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetControlAnimationIndexEvent:new(self.vehicle, self.index), nil, connection, self.vehicle);
    end;
end;

function SetControlAnimationIndexEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteInt8(streamId, self.index);
end;

function SetControlAnimationIndexEvent.sendEvent(vehicle, index, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetControlAnimationIndexEvent:new(vehicle, index), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetControlAnimationIndexEvent:new(vehicle, index));
		end;
	end;
end;

SetAccumulatedFruitTypeEvent = {};
SetAccumulatedFruitTypeEvent_mt = Class(SetAccumulatedFruitTypeEvent, Event);

InitEventClass(SetAccumulatedFruitTypeEvent, "SetAccumulatedFruitTypeEvent");

function SetAccumulatedFruitTypeEvent:emptyNew()
    local self = Event:new(SetAccumulatedFruitTypeEvent_mt);
    self.className="SetAccumulatedFruitTypeEvent";
    return self;
end;

function SetAccumulatedFruitTypeEvent:new(vehicle, fruitType)
    local self = SetAccumulatedFruitTypeEvent:emptyNew()
    self.vehicle = vehicle;
	self.fruitType = fruitType;
    return self;
end;

function SetAccumulatedFruitTypeEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);

	self.fruitType = streamReadInt8(streamId);
	if self.vehicle ~= nil then
		self.vehicle.accumulatedFruitType = self.fruitType;
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetAccumulatedFruitTypeEvent:new(self.vehicle, self.fruitType), nil, connection, self.vehicle);
    end;
end;

function SetAccumulatedFruitTypeEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteInt8(streamId, self.fruitType);
end;

function SetAccumulatedFruitTypeEvent.sendEvent(vehicle, fruitType, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetAccumulatedFruitTypeEvent:new(vehicle, fruitType), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetAccumulatedFruitTypeEvent:new(vehicle, fruitType));
		end;
	end;
end;

SetControlBeltIndexEvent = {};
SetControlBeltIndexEvent_mt = Class(SetControlBeltIndexEvent, Event);

InitEventClass(SetControlBeltIndexEvent, "SetControlBeltIndexEvent");

function SetControlBeltIndexEvent:emptyNew()
    local self = Event:new(SetControlBeltIndexEvent_mt);
    self.className="SetControlBeltIndexEvent";
    return self;
end;

function SetControlBeltIndexEvent:new(vehicle, index)
    local self = SetControlBeltIndexEvent:emptyNew()
    self.vehicle = vehicle;
	self.index = index;
    return self;
end;

function SetControlBeltIndexEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);

	self.index = streamReadInt8(streamId);
	if self.vehicle ~= nil then
		self.vehicle:setControlBeltIndex(self.index, true);
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetControlBeltIndexEvent:new(self.vehicle, self.index), nil, connection, self.vehicle);
    end;
end;

function SetControlBeltIndexEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteInt8(streamId, self.index);
end;

function SetControlBeltIndexEvent.sendEvent(vehicle, index, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetControlBeltIndexEvent:new(vehicle, index), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetControlBeltIndexEvent:new(vehicle, index));
		end;
	end;
end;

SetUnloadingEvent = {};
SetUnloadingEvent_mt = Class(SetUnloadingEvent, Event);

InitEventClass(SetUnloadingEvent, "SetUnloadingEvent");

function SetUnloadingEvent:emptyNew()
    local self = Event:new(SetUnloadingEvent_mt);
    self.className="SetUnloadingEvent";
    return self;
end;

function SetUnloadingEvent:new(vehicle, unloading, direction)
    local self = SetUnloadingEvent:emptyNew()
    self.vehicle = vehicle;
	self.unloading = unloading;
	self.direction = direction;
    return self;
end;

function SetUnloadingEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);

	self.unloading = streamReadBool(streamId);
	self.direction = streamReadInt8(streamId);
	if self.vehicle ~= nil then
		self.vehicle:setUnloading(self.unloading, self.direction, true);
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetUnloadingEvent:new(self.vehicle, self.unloading, self.direction), nil, connection, self.vehicle);
    end;
end;

function SetUnloadingEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteBool(streamId, self.unloading);
	streamWriteInt8(streamId, self.direction);
end;

function SetUnloadingEvent.sendEvent(vehicle, unloading, direction, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetUnloadingEvent:new(vehicle, unloading, direction), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetUnloadingEvent:new(vehicle, unloading, direction));
		end;
	end;
end;


SetAnimationStateEvent = {};
SetAnimationStateEvent_mt = Class(SetAnimationStateEvent, Event);

InitEventClass(SetAnimationStateEvent, "SetAnimationStateEvent");

function SetAnimationStateEvent:emptyNew()
    local self = Event:new(SetAnimationStateEvent_mt);
    self.className="SetAnimationStateEvent";
    return self;
end;

function SetAnimationStateEvent:new(vehicle, animationState)
    local self = SetAnimationStateEvent:emptyNew()
    self.vehicle = vehicle;
	self.animationState = animationState;
    return self;
end;

function SetAnimationStateEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);

	self.animationState = streamReadInt8(streamId);
	if self.vehicle ~= nil then
		self.vehicle:setAnimationState(self.animationState, true);
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetAnimationStateEvent:new(self.vehicle, self.animationState), nil, connection, self.vehicle);
    end;
end;

function SetAnimationStateEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteInt8(streamId, self.animationState);
end;

function SetAnimationStateEvent.sendEvent(vehicle, animationState, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetAnimationStateEvent:new(vehicle, animationState), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetAnimationStateEvent:new(vehicle, animationState));
		end;
	end;
end;
