--
-- PowerShaft
-- Specialization for a powershaft
--
-- @author  Stefan Maurus	
-- @date  2015-09-04

-- Copyright © Stefan Maurus, www.stefanmaurus.de

PowerShaft = {};

function PowerShaft.prerequisitesPresent(specializations)
    return true;
end;

function PowerShaft:load(savegame)
	self.powershaft = {};
	self.powershaft.node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.powershaft#index"));
	self.powershaft.translationPart = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.powershaft#translationPart"));	
	_,_,self.powershaft.translationPartStartZ = getTranslation(self.powershaft.translationPart);
	self.powershaft.translationPartOffset = getXMLFloat(self.xmlFile, "vehicle.powershaft#translationPartOffset");	
	self.powershaft.linkNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.powershaft#linkNode"));		
	self.powershaft.detachNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.powershaft#detachNode"));		
	self.powershaft.isAttached = false;
end;

function PowerShaft:delete()
end;

function PowerShaft:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PowerShaft:keyEvent(unicode, sym, modifier, isDown)
end;

function PowerShaft:readStream(streamId, connection)
end;

function PowerShaft:writeStream(streamId, connection) 	
end;

function PowerShaft:update(dt)	
	if self:getIsActive() then		
		if self.powershaft.isAttached then		
			local x1, y1, z1 = getWorldTranslation(self.powershaft.node);
			local x2, y2, z2 = getWorldTranslation(getChildAt(self.powershaft.linkNode, 0));
			local x, y, z = worldDirectionToLocal(getParent(self.powershaft.node), x2-x1, y2-y1, z2-z1);
			setDirection(self.powershaft.node, x, y, z, 0, 1, 0);
			
			local distance = Utils.vector3Length(x1-x2, y1-y2, z1-z2);
			setTranslation(self.powershaft.translationPart, 0, 0, distance+self.powershaft.translationPartOffset);		
		end;		
	end;
end;

function PowerShaft:updateTick(dt)
end;

function PowerShaft:draw()
end;

function PowerShaft:onAttach(attacherVehicle, jointDescIndex)
	local outputNode = nil;
	if attacherVehicle ~= nil then
		if attacherVehicle.attacherJoints[jointDescIndex].ptoOutput ~= nil then
			outputNode = attacherVehicle.attacherJoints[jointDescIndex].ptoOutput.node;
		end;
	end;

	if outputNode == nil and (attacherVehicle.lowTrailerIsAttached ~= nil or attacherVehicle.isFastCoupler ~= nil) then
		local tractor = attacherVehicle.attacherVehicle;
		if tractor ~= nil then
			local implement = tractor:getImplementByObject(attacherVehicle);
			if implement ~= nil then
				if tractor.attacherJoints[implement.jointDescIndex].ptoOutput then
					outputNode = tractor.attacherJoints[implement.jointDescIndex].ptoOutput.node;
				end;
			end;
		end;
	end;
	
	if outputNode ~= nil then
		link(outputNode, self.powershaft.linkNode);
		setRotation(self.powershaft.linkNode, 0, math.rad(180), 0);
		self.powershaft.isAttached = true;
	end;
end;

function PowerShaft:onDetach(attacherVehicle, jointDescIndex)
	link(self.powershaft.detachNode, self.powershaft.linkNode);
	setRotation(self.powershaft.node, 0, 0, 0);
	setRotation(self.powershaft.linkNode, 0, 0, 0);	
	setTranslation(self.powershaft.translationPart, 0, 0, self.powershaft.translationPartStartZ);
	self.powershaft.isAttached = false;	
end;