--
-- WorkLights
-- Specialization for separate turn on of worklights on attachables
--
-- @author  Stefan Maurus
-- @date  2016-04-03

-- Copyright © Stefan Maurus, www.stefanmaurus.de


WorkLights = {};

function WorkLights.prerequisitesPresent(specializations)
    return true;
end;

function WorkLights:load(savegame)
	self.setWorkLight = WorkLights.setWorkLight
	self.workLightTurnedOn = false;
	self.workLightType = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.workLight#type"), 4);

	self.lightsTypesMask_backup = self.lightsTypesMask;
end;

function WorkLights:delete()
end;

function WorkLights:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WorkLights:keyEvent(unicode, sym, modifier, isDown)
end;

function WorkLights:setBeltAnimationTime(name, t)
end;

function WorkLights:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.WORKLIGHT_ONOFF) then
			self:setWorkLight(not self.workLightTurnedOn);
			
			SoundUtil.playSample(g_currentMission.sampleToggleLights, 1, 0, 1);
		end;
	end;

	if self.lightsTypesMask_backup ~= self.lightsTypesMask then --if attacher vehicle light changes
		self:setWorkLight(self.workLightTurnedOn);
		self.lightsTypesMask_backup = self.lightsTypesMask;
	end;
end;

function WorkLights:updateTick(dt)
end;

function WorkLights:draw()
	if self.workLightTurnedOn then
		g_currentMission:addHelpButtonText(g_i18n:getText("WORKLIGHT_OFF"), InputBinding.WORKLIGHT_ONOFF, nil, GS_PRIO_HIGH);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("WORKLIGHT_ON"), InputBinding.WORKLIGHT_ONOFF, nil, GS_PRIO_HIGH);
	end;
end;

function WorkLights:setWorkLight(state, noEventSend)
	self.workLightTurnedOn = state;

	if state then
		local value = 2^self.workLightType;
		local isActive = bitAND(self.lightsTypesMask, value) ~= 0;
		if not isActive then
			self:setLightsTypesMask(self.lightsTypesMask+value);
		end;
	else
		local value = 2^self.workLightType;
		local isActive = bitAND(self.lightsTypesMask, value) ~= 0;
		if isActive then
			self:setLightsTypesMask(self.lightsTypesMask-value);
		end;
	end;
	
	SetWorkLightEvent.sendEvent(self, self.workLightTurnedOn, noEventSend);
end;

function WorkLights:onDetach()
	self:setWorkLight(false);
end;


SetWorkLightEvent = {};
SetWorkLightEvent_mt = Class(SetWorkLightEvent, Event);

InitEventClass(SetWorkLightEvent, "SetWorkLightEvent");

function SetWorkLightEvent:emptyNew()
    local self = Event:new(SetWorkLightEvent_mt);
    self.className="SetWorkLightEvent";
    return self;
end;

function SetWorkLightEvent:new(vehicle, turnedOn)
    local self = SetWorkLightEvent:emptyNew()
    self.vehicle = vehicle;
	self.turnedOn = turnedOn;
    return self;
end;

function SetWorkLightEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);

	self.turnedOn = streamReadBool(streamId);
	if self.vehicle ~= nil then
		self.vehicle:setWorkLight(self.turnedOn, true);
	end;

	if not connection:getIsServer() then
        g_server:broadcastEvent(SetWorkLightEvent:new(self.vehicle, self.turnedOn), nil, connection, self.vehicle);
    end;
end;

function SetWorkLightEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.turnedOn);
end;

function SetWorkLightEvent.sendEvent(vehicle, turnedOn, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetWorkLightEvent:new(vehicle, turnedOn), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetWorkLightEvent:new(vehicle, turnedOn));
		end;
	end;
end;